@@ -22,6 +22,7 @@
 
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-axis.h>
+#include <goffice/graph/gog-chart.h>
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/graph/gog-label.h>
 #include <goffice/graph/gog-outlined-object.h>
@@ -32,6 +33,7 @@
 #include <goffice/graph/gog-reg-curve.h>
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/data/go-data.h>
+#include <goffice/utils/go-persist.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
@@ -121,10 +123,13 @@ gog_text_init_style (GogStyledObject *gso, GogStyle *style)
 	 * in GogTheme, but it's not possible without breaking graph persistence
 	 * compatibility */
 	parent = gog_object_get_parent (GOG_OBJECT (gso));
-	if (IS_GOG_AXIS (parent) &&
-	    gog_axis_get_atype (GOG_AXIS (parent)) == GOG_AXIS_Y &&
-	    style->text_layout.auto_angle) 
-		style->text_layout.angle = 90.0;
+	if (IS_GOG_AXIS (parent)) {
+		GogChart *chart = GOG_CHART (gog_object_get_parent (parent));
+		if (!gog_chart_is_3d (chart)
+		    && gog_axis_get_atype (GOG_AXIS (parent)) == GOG_AXIS_Y
+		    && style->text_layout.auto_angle)
+			style->text_layout.angle = 90.0;
+	}
 }
 
 static void
@@ -143,7 +148,7 @@ gog_text_class_init (GogTextClass *klass)
 			_("Allow markup"),
 			_("Support basic html-ish markup"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	gog_klass->view_type		= gog_text_view_get_type ();
 	style_klass->init_style 	= gog_text_init_style;
@@ -421,13 +426,13 @@ gog_reg_eqn_class_init (GogObjectClass *gog_klass)
 			_("Show equation"),
 			_("Show the equation on the graph"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, REG_EQN_SHOW_R2,
 		g_param_spec_boolean ("show-r2", 
 			_("Show coefficient"),
 			_("Show the correlation coefficient on the graph"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 }
 
 static void
@@ -495,7 +500,7 @@ gog_text_view_render (GogView *view, GogViewAllocation const *bbox)
 			rect = view->allocation;
 			rect.w = aabr.w + 2. * outline + pad_x;
 			rect.h = aabr.h + 2. * outline + pad_y;
-			gog_renderer_draw_sharp_rectangle (view->renderer, &rect);
+			gog_renderer_draw_rectangle (view->renderer, &rect);
 		}
 		gog_renderer_draw_text (view->renderer, str,
 					&view->residual, GTK_ANCHOR_NW, FALSE);
