@@ -119,6 +119,7 @@ typedef struct {
 					 GogEditor *editor,
 					 GogDataAllocator *dalloc, 
 					 GOCmdContext *cc);
+	void	     (*document_changed)(GogObject *obj, GODoc *doc);
 
 	/* signals */
 	void (*changed)		(GogObject *obj, gboolean size);
@@ -138,7 +139,6 @@ typedef struct {
 #define IS_GOG_OBJECT_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), GOG_OBJECT_TYPE))
 #define GOG_OBJECT_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), GOG_OBJECT_TYPE, GogObjectClass))
 
-#define GOG_PARAM_PERSISTENT	(1 << (G_PARAM_USER_SHIFT+0))
 #define GOG_PARAM_FORCE_SAVE	(1 << (G_PARAM_USER_SHIFT+1))	/* even if the value == default */
 #define GOG_PARAM_POSITION	(1 << (G_PARAM_USER_SHIFT+2))	/* position parameters */
 
@@ -198,6 +198,8 @@ void 	 gog_object_register_roles	  (GogObjectClass *klass,
 					   GogObjectRole const *roles, unsigned int n_roles);
 void 	 gog_object_request_editor_update (GogObject *obj);
 
+void	 gog_object_document_changed	  (GogObject *obj, GODoc *doc);
+
 G_END_DECLS
 
 #endif /* GOG_OBJECT_H */
