@@ -21,6 +21,7 @@
 #ifndef GOG_RENDERER_H
 #define GOG_RENDERER_H
 
+#include <goffice/goffice-features.h>
 #include <goffice/graph/goffice-graph.h>
 #include <goffice/utils/go-geometry.h>
 #include <goffice/utils/go-image.h>
@@ -28,15 +29,13 @@
 #include <glib-object.h>
 
 #include <gsf/gsf.h>
-#include <libart_lgpl/libart.h>
 
-#if 1 /* def GOFFICE_WITH_GTK breaks without goffice-config.h included */
+#ifdef GOFFICE_WITH_GTK
 #include <gtk/gtkenums.h>
 #include <goffice/gtk/goffice-gtk.h>
 #include <gdk/gdk.h>
 #else
-/* VILE HACK VILE HACK VILE HACK */
-typedef struct HACK_GdkPixbuf GdkPixbuf;
+typedef struct _GdkPixbuf GdkPixbuf;
 typedef enum
 {
   GTK_ANCHOR_CENTER,
@@ -57,7 +56,6 @@ typedef enum
   GTK_ANCHOR_W		= GTK_ANCHOR_WEST,
   GTK_ANCHOR_E		= GTK_ANCHOR_EAST
 } GtkAnchorType;
-/* VILE HACK VILE HACK VILE HACK */
 #endif
 
 #include <cairo.h>
@@ -93,7 +91,7 @@ double gog_renderer_pt2r_x   	  	(GogRenderer const *r, double d);
 double gog_renderer_pt2r_y   	  	(GogRenderer const *r, double d);
 double gog_renderer_pt2r   	  	(GogRenderer const *r, double d);
 
-double gog_renderer_get_hairline_width_pts	(GogRenderer const *r);
+double gog_renderer_get_hairline_width_pts	(GogRenderer const *rend);
 
 void  gog_renderer_stroke_serie    	(GogRenderer *renderer, GOPath const *path);
 void  gog_renderer_fill_serie		(GogRenderer *renderer, GOPath const *path, GOPath const *close_path);
@@ -102,8 +100,13 @@ void  gog_renderer_draw_shape 		(GogRenderer *renderer, GOPath const *path);
 void  gog_renderer_stroke_shape		(GogRenderer *renderer, GOPath const *path);
 void  gog_renderer_fill_shape 		(GogRenderer *renderer, GOPath const *path);
 
+void  gog_renderer_draw_circle		(GogRenderer *rend, double x, double y, double r);
+void  gog_renderer_stroke_circle 	(GogRenderer *rend, double x, double y, double r);
+void  gog_renderer_fill_circle	 	(GogRenderer *rend, double x, double y, double r);
+
 void  gog_renderer_draw_rectangle	(GogRenderer *rend, GogViewAllocation const *rect);
 void  gog_renderer_stroke_rectangle 	(GogRenderer *rend, GogViewAllocation const *rect);
+void  gog_renderer_fill_rectangle 	(GogRenderer *rend, GogViewAllocation const *rect);
 
 void  gog_renderer_draw_grip	  		(GogRenderer *renderer, double x, double y);
 void  gog_renderer_draw_selection_rectangle	(GogRenderer *renderer, GogViewAllocation const *rectangle);
@@ -131,24 +134,20 @@ void  gog_renderer_push_clip 	  	(GogRenderer *rend, GOPath const *path);
 void  gog_renderer_push_clip_rectangle 	(GogRenderer *rend, double x, double y, double w, double h);
 void  gog_renderer_pop_clip       	(GogRenderer *rend);
 
-/* utilities for cairo/libart transition */
-GogRenderer 	*gog_renderer_new_for_pixbuf 	(GogGraph *graph);
-gboolean	 gog_renderer_update		(GogRenderer *renderer, double w, double h, double zoom);
-GdkPixbuf 	*gog_renderer_get_pixbuf 	(GogRenderer *renderer);
 
-GogRenderer 	*gog_renderer_new_for_format 	(GogGraph *graph, GOImageFormat format);
+/* Rendering with image cache */
+gboolean	 gog_renderer_update		(GogRenderer *renderer, double w, double h);
+cairo_surface_t *gog_renderer_get_cairo_surface	(GogRenderer *renderer);
+GdkPixbuf 	*gog_renderer_get_pixbuf 	(GogRenderer *renderer);
+void  		 gog_renderer_request_update 	(GogRenderer *renderer);
+
+/* One time rendering */
+gboolean	 gog_renderer_render_to_cairo 	(GogRenderer *renderer, cairo_t *cairo,
+						 double width, double height);
 gboolean 	 gog_renderer_export_image 	(GogRenderer *renderer, GOImageFormat format,
 						 GsfOutput *output, double x_dpi, double y_dpi);
 
-void  gog_renderer_request_update (GogRenderer *r);
-
-/* Deprecated API */
-void  gog_renderer_draw_sharp_path	(GogRenderer *rend, ArtVpath *path);
-void  gog_renderer_draw_sharp_polygon   (GogRenderer *rend, ArtVpath *path, gboolean narrow);
-
-void  gog_renderer_draw_path      	(GogRenderer *rend, ArtVpath const *path);
-void  gog_renderer_draw_polygon   	(GogRenderer *rend, ArtVpath const *path, gboolean narrow);
-void  gog_renderer_draw_bezier_path     (GogRenderer *rend, ArtBpath const *path);
+GogRenderer 	*gog_renderer_new 		(GogGraph *graph);
 
 G_END_DECLS
 
