@@ -11,6 +11,7 @@
 
 #include <goffice/goffice-config.h>
 #include "datetime.h"
+#include <goffice/math/go-math.h>
 
 #include <math.h>
 #include <string.h>
@@ -133,12 +134,14 @@ time_t
 datetime_serial_to_timet (int serial, GODateConventions const *conv)
 {
 	GDate gd;
-	struct tm tm;
 
 	datetime_serial_to_g (&gd, serial, conv);
-	g_date_to_struct_tm (&gd, &tm);
-
-	return mktime (&tm);
+	if (g_date_valid (&gd)) {
+		struct tm tm;
+		g_date_to_struct_tm (&gd, &tm);
+		return mktime (&tm);
+	} else
+		return -1;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -206,7 +209,7 @@ datetime_g_years_between (GDate const *date1, GDate const *date2)
 int
 datetime_weeknum (GDate const *date, int method)
 {
-	int res;
+	int res = -1;
 
 	g_return_val_if_fail (g_date_valid (date), -1);
 	g_return_val_if_fail (method == WEEKNUM_METHOD_SUNDAY ||
@@ -221,7 +224,6 @@ datetime_weeknum (GDate const *date, int method)
 		res = g_date_get_monday_week_of_year (date); break;
 	case WEEKNUM_METHOD_ISO:
 		res = g_date_get_iso8601_week_of_year (date); break;
-	default: res = -1;
 	}
 
 	return res;
@@ -500,6 +502,56 @@ gnm_date_convention_base (GODateConventions const *conv)
 	return conv->use_1904 ? 1904 : 1900;
 }
 
+const GODateConventions *
+go_date_conv_from_str (const char *s)
+{
+	static const GODateConventions apple1904 = { TRUE };
+	static const GODateConventions lotus1900 = { FALSE };
+
+	g_return_val_if_fail (s != NULL, NULL);
+
+	if (strcmp (s, "Apple:1904") == 0)
+		return &apple1904;
+
+	if (strcmp (s, "Lotus:1900") == 0)
+		return &lotus1900;
+
+	return NULL;
+}
+
+gboolean
+go_date_conv_equal (const GODateConventions *a, const GODateConventions *b)
+{
+	g_return_val_if_fail (a != NULL, FALSE);
+	g_return_val_if_fail (b != NULL, FALSE);
+
+	return a->use_1904 == b->use_1904;
+}
+
+double
+go_date_conv_translate (double f,
+			const GODateConventions *src,
+			const GODateConventions *dst)
+{
+	g_return_val_if_fail (src != NULL, f);
+	g_return_val_if_fail (dst != NULL, f);
+
+	if (!go_finite (f) || go_date_conv_equal (src, dst))
+		return f;
+
+	if (dst->use_1904) {
+		if (f < date_serial_19000228 + 1)
+			f += 1;
+		f -= 1462;
+	} else {
+		f += 1462;
+		if (f < date_serial_19000228 + 2)
+			f -= 1;
+	}
+
+	return f;
+}
+
 /* ------------------------------------------------------------------------- */
 
 static char *
