@@ -287,10 +287,10 @@ go_url_resolve_relative (char const *ref_uri, char const *rel_uri)
 
 	len = strlen (ref_uri);
 
-	/* FIXME: This doesn't work if rel_uri starts with a slash.  */
+	/* FIXME: This doesn't work if ref_uri starts with a slash.  */
 
 	uri = g_new (char, len + strlen (rel_uri) + 1);
-	memcpy (uri, rel_uri, len + 1);
+	memcpy (uri, ref_uri, len + 1);
 	while (len > 0 && uri[len - 1] != '/')
 		len--;
 	if (len == 0) {
@@ -691,6 +691,7 @@ go_get_file_permissions (char const *uri)
 		file_permissions->owner_write   = ((mode & S_IWUSR) != 0);
 		file_permissions->owner_execute = ((mode & S_IXUSR) != 0);
 
+#ifndef G_OS_WIN32
 		/* Group  Permissions */
 		file_permissions->group_read    = ((mode & S_IRGRP) != 0);
 		file_permissions->group_write   = ((mode & S_IWGRP) != 0);
@@ -700,6 +701,14 @@ go_get_file_permissions (char const *uri)
 		file_permissions->others_read    = ((mode & S_IROTH) != 0);
 		file_permissions->others_write   = ((mode & S_IWOTH) != 0);
 		file_permissions->others_execute = ((mode & S_IXOTH) != 0);
+#else
+		file_permissions->group_read    =
+		file_permissions->group_write   =
+		file_permissions->group_execute =
+		file_permissions->others_read    =
+		file_permissions->others_write   =
+		file_permissions->others_execute = FALSE;
+#endif /* G_OS_WIN32 */
 	}
 		
 	if (info)
@@ -726,6 +735,7 @@ go_set_file_permissions (char const *uri, GOFilePermissions * file_permissions)
 	if (file_permissions->owner_execute == TRUE)
 		permissions |= S_IXUSR;
 
+#ifndef G_OS_WIN32
 	/* Set group permissions */
 	if (file_permissions->group_read == TRUE)
 		permissions |= S_IRGRP;
@@ -745,6 +755,7 @@ go_set_file_permissions (char const *uri, GOFilePermissions * file_permissions)
 
 	if (file_permissions->others_execute == TRUE)
 		permissions |= S_IXOTH;
+#endif
 
 	/* Try setting unix mode */
 	g_file_set_attribute_uint32 (file,
@@ -814,10 +825,6 @@ go_file_get_date (char const *uri, GOFileDateType type)
 		tm = (time_t) g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_CHANGED);
 		break;
 	}
-	if (error)
-		g_free (error);
-	else {
-	}
 		
 	if (info)
 		g_object_unref (info);
@@ -1098,7 +1105,8 @@ go_get_mime_type (gchar const *uri)
 		mime_type = g_content_type_get_mime_type (content_type);
 		g_free (content_type);
 	}
-	g_object_unref (info);
+	if (info)
+		g_object_unref (info);
 	if (mime_type)
 		return mime_type;
 	return g_strdup ("application/octet-stream");
